# THIS IS AN AUTOMATICALLY GENERATED FILE.
# CHANGES TO IT WILL BE DESTROYED BY Dist::Zilla.

[TestingAndDebugging::RequireUseStrict]
equivalent_modules = strictures

[-OTRS::RequireCamelCase]
[-OTRS::RequireParensWithMethods]
[-ValuesAndExpressions::ProhibitAccessOfPrivateData]
[-OTRS::ProhibitLowPrecedenceOps]

